"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const command_1 = require("@oclif/command/lib/command");
const flags = require("@oclif/command/lib/flags");
const dirTree = require("directory-tree");
const fs = require("fs");
const path = require("path");
const copy = require("recursive-copy");
const rimraf = require("rimraf");
const through = require("through2");
const conf_1 = require("../services/conf");
const doc_1 = require("../services/doc");
const logger_1 = require("../services/logger");
const markdown_1 = require("../services/markdown");
const message_1 = require("../services/message");
const render_1 = require("../services/render");
const track_1 = require("../services/track");
const { resolve, join } = path;
class DocsBase extends command_1.default {
    constructor() {
        super(...arguments);
        this.verbose = false;
    }
    get navs() {
        const navs = [
            { title: '文档', main: true },
            { title: 'FAQ', url: 'FAQ.html' },
        ];
        if (conf_1.default.isContainer) {
            const nav = { title: '速查', url: 'cheatsheet.html' };
            navs.splice(1, 0, nav);
            return navs;
        }
        return navs;
    }
    get userStyles() {
        const { docs = {} } = this.localConfig;
        return docs.userStyles || [];
    }
    get userScripts() {
        const { docs = {} } = this.localConfig;
        return docs.userScripts || [];
    }
    get frameworkScripts() {
        const { framework = 'vue' } = this.localConfig;
        if (!conf_1.default.frameworks[framework]) {
            throw Error(message_1.MCOMMON.UNSUPPORTED_FRAMEWORK);
        }
        const { cdn = [] } = conf_1.default.frameworks[framework];
        let scripts = '';
        cdn.forEach(url => {
            scripts += `<script src="${url}"><script>`;
        });
        return scripts;
    }
    get options() {
        return {
            overwrite: true,
            rename(filePath) {
                if (filePath.endsWith('.md')) {
                    const dirname = path.dirname(filePath);
                    const filename = path.basename(filePath, '.md');
                    return join(dirname, `${filename}.html`);
                }
                return filePath;
            },
            filter(src) {
                src = join(conf_1.default.docsDir, src);
                if (DocsBase.excludeDirRule.test(src)) {
                    return false;
                }
                return true;
            },
            transform: () => {
                return through((chunk, _enc, done) => {
                    const content = markdown_1.default.render(chunk.toString());
                    const { loadTemplate: load } = render_1.default;
                    const index = load('docs/index');
                    const style = load('docs/style');
                    const layout = load('docs/layout');
                    const { docs = {} } = this.localConfig || {};
                    const output = render_1.default.render(index, {
                        title: docs.title || '',
                        baseUrl: docs.url || '',
                        content,
                        navs: JSON.stringify(this.navs),
                        pages: JSON.stringify(this.pages),
                        userStyles: this.userStyles,
                        userScripts: this.userScripts,
                    }, ['<%', '%>'], {
                        style,
                        layout,
                        frameworkScripts: this.frameworkScripts,
                    });
                    done(null, output);
                });
            }
        };
    }
    async init() {
        // @ts-ignore
        const { flags } = this.parse(this.constructor);
        this.verbose = flags.verbose;
        await this.config.runHook('checkUpdate', {});
        const { appConfPath, rdcConfPath, RdTypes } = conf_1.default;
        if (fs.existsSync(appConfPath)) {
            conf_1.default.rdType = RdTypes.Application;
        }
        else if (fs.existsSync(rdcConfPath)) {
            conf_1.default.rdType = RdTypes.Container;
        }
        if (!conf_1.default.rdType) {
            throw Error(message_1.MDOCS.UNRECOGNIZED);
        }
        if (!fs.existsSync(conf_1.default.docsDir)) {
            throw Error(message_1.MDOCS.MISSING_DOCS_DIR);
        }
        const homepagePath = resolve(conf_1.default.docsDir, 'index.md');
        const faqPath = resolve(conf_1.default.docsDir, 'faq.md');
        if (!fs.existsSync(homepagePath) || !fs.existsSync(faqPath)) {
            throw Error(message_1.MDOCS.MISSING_REQUIRED_MD);
        }
        if (conf_1.default.rdType === RdTypes.Container) {
            this.localConfig = require(rdcConfPath);
        }
        else {
            this.localConfig = require(appConfPath);
        }
        this.pages = this.getPages();
        await this.render();
        await this.preRun();
        process.on('SIGINT', () => {
            rimraf.sync(conf_1.default.docsPagesDir);
            process.exit();
        });
    }
    async preRun() {
        if (conf_1.default.isApp) {
            return;
        }
        const { loadTemplate: load } = render_1.default;
        const index = load('docs/index');
        const style = load('docs/style');
        const layout = load('docs/layout');
        const cheatsheet = load('docs/cheatsheet');
        const { docs = {}, mappings = [] } = this.localConfig || {};
        const content = render_1.default.render(cheatsheet, {
            deps: await doc_1.default.getPkgDeps(),
            lint: doc_1.default.getLintModule(),
            mappings,
        });
        const output = render_1.default.render(index, {
            title: docs.title || '',
            baseUrl: docs.url || '',
            content,
            navs: JSON.stringify(this.navs),
            pages: JSON.stringify(this.pages),
            userStyles: this.userStyles,
            userScripts: this.userScripts,
        }, ['<%', '%>'], {
            style,
            layout,
            frameworkScripts: this.frameworkScripts,
        });
        fs.writeFileSync(resolve(conf_1.default.docsPagesDir, 'cheatsheet.html'), output, { encoding: 'utf-8' });
    }
    async postRun() { }
    async catch(e) {
        logger_1.logger.error(e.message);
        logger_1.debug('stack: %O', e);
        this.exit(1);
    }
    async finally(e) {
        if (!e) {
            await this.postRun();
        }
    }
    getPages() {
        let pages = [];
        const files = dirTree(conf_1.default.docsDir, {
            extensions: /\.md$/,
            exclude: DocsBase.excludeDirRule,
        });
        files.children.forEach(file => {
            if (file.type === 'file') {
                let content = fs.readFileSync(file.path).toString();
                markdown_1.default.render(content);
                const { meta = {} } = markdown_1.default;
                pages.push({
                    title: meta.title || file.name,
                    subTitle: meta.subTitle || '',
                    url: `${file.name.slice(0, -3)}.html`,
                    order: meta.order || 0
                });
            }
            if (file.type === 'directory') {
                let category;
                let categoryOrder;
                const page = {
                    title: file.name,
                    children: file.children.map(child => {
                        let content = fs.readFileSync(child.path).toString();
                        markdown_1.default.render(content);
                        const { meta = {} } = markdown_1.default;
                        if (meta.category) {
                            if (meta.category) {
                                category = meta.category;
                            }
                            if (meta.categoryOrder) {
                                categoryOrder = meta.categoryOrder;
                            }
                        }
                        return {
                            title: meta.title || child.name,
                            subTitle: meta.subTitle || '',
                            url: `${file.name}/${child.name.slice(0, -3)}.html`,
                            order: meta.order || 0
                        };
                    }),
                    order: 0,
                };
                page.title = category || file.name;
                page.order = categoryOrder || 0;
                page.children.sort((prev, next) => {
                    return next.order - prev.order;
                });
                pages.push(page);
            }
        });
        pages.sort((prev, next) => {
            return next.order - prev.order;
        });
        return pages;
    }
    async render() {
        const { docsDir, docsPagesDir } = conf_1.default;
        await copy(docsDir, docsPagesDir, this.options);
    }
}
DocsBase.flags = {
    verbose: flags.boolean({ char: 'v', required: false, description: 'show verbose logs' }),
};
DocsBase.excludeDirRule = /^_docs\/[^\/]+\/(?!([^\/]*\.md$))+/;
tslib_1.__decorate([
    track_1.TError({ conf: conf_1.default }),
    track_1.TStart
], DocsBase.prototype, "init", null);
exports.default = DocsBase;
