"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const semver = require("semver");
const base_1 = require("../base");
const conf_1 = require("../services/conf");
const docker_1 = require("../services/docker");
const logger_1 = require("../services/logger");
const message_1 = require("../services/message");
const track_1 = require("../services/track");
const validate_1 = require("../services/validate");
class Publish extends base_1.default {
    async preInit() {
        await this.config.runHook('checkUpdate', {});
        const { flags } = this.parse(Publish);
        this.tag = flags.tag;
        this.level = flags.increment;
        this.preid = flags.preid;
        if (!this.level && !this.tag && !this.preid) {
            throw Error(message_1.MCOMMON.UNRECOGNIZED_VERSION_FLAG);
        }
        if (this.tag === 'latest') {
            throw Error(message_1.MCOMMON.WRONG_PUBLISH_TAG);
        }
        this.rdcConf = await validate_1.validateRdc(true);
        return flags;
    }
    async initialize() {
        const { docker: dockerConf } = this.rdcConf;
        this.rdc = dockerConf.tag;
        await docker_1.default.genDockerFile4Publish(conf_1.default.dockerWorkDirRoot, this.rdcConf.extends || 'node:latest', conf_1.default.localCacheDir);
    }
    async run() {
        track_1.uploadHubble({ cmd: 'publish' });
        const arr = this.rdc.split(':');
        const name = arr[0];
        let version = arr[1];
        if (this.preid) {
            this.level = this.level || 'prerelease';
            version = semver.inc(version, this.level, this.preid);
        }
        else if (this.level) {
            version = semver.inc(version, this.level);
        }
        else if (this.tag) {
            version = this.tag;
        }
        logger_1.debug(`publish version is ${version}`);
        const image = `${name}:${version}`;
        await docker_1.default.build(image, conf_1.default.localCacheDir, true, 'Dockerfile', '..');
        await docker_1.default.tag(image, image);
        await docker_1.default.tag(image, `${name}:latest`);
        await docker_1.default.push(name, version);
        this.tag = image;
    }
    async postRun() {
        logger_1.logger.log(`Published ${this.tag} successfully`);
    }
}
Publish.examples = [
    '$ rde publish -t <semver-like version>',
    '$ rde publish -i prerelease --preid beta',
    '$ rde publish --preid beta',
];
Publish.flags = Object.assign({}, base_1.default.flags, { increment: command_1.flags.string({
        char: 'i',
        description: `Increment a version by the specified level.
        Level can be one of: major, minor, patch, premajor, preminor,
        prepatch, or prerelease.  Default level is \'patch\'. Only one version may be specified.`,
        required: false,
    }), preid: command_1.flags.string({
        description: 'Identifier to be used to prefix premajor, preminor, prepatch or prerelease version increments.',
        required: false,
    }), tag: command_1.flags.string({
        char: 't',
        description: 'as your image tag for pushing to docker hub, you should use increment just in case',
        required: false,
    }) });
exports.default = Publish;
